# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimList
from driverng.astm import HprimMultiple
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.hl7_ihe_pam_adt")


class HL7IHEPAMADTMSH(HeaderSegment):
    sending_application = HL7String(3)
    sending_facility = HL7String(4)
    receiving_application = HL7String(5)
    receiving_facility = HL7String(6)
    message_date = HL7Date(7)
    message_code = HL7String((9, 1))
    trigger_event = HL7String((9, 2))
    message_structure = HL7String((9, 3))
    message_control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version_id = HL7String((12, 1))
    internationalization_code = HL7String((12, 2))
    international_version_id = HL7String((12, 3))
    sequence_number = HL7String(13)
    accept_acknowledgement_type = HL7String(15)
    application_acknowledgement_type = HL7String(16)
    country_code = HL7String(17)
    character_set = HL7String(18)
    principal_language = HL7String(19)
    message_profile_identifier = HL7String(21)

    events = HL7Foreign("EVN", "header")


class HL7IHEPAMADTEVN(Segment):
    recorded_date = HL7Date(3)
    event_occured = HL7String(7)

    patients = HL7Foreign("PID", "patient")


class HL7IHEPAMADTPID(Segment):
    patient_identifier = HprimList(
        4,
        basefield=HprimMultiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HprimMultiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
            assigning_facility=HL7String(6),
            effective_date=HL7Date(7),
        ),
    )
    identity = HprimList(
        6,
        basefield=HprimMultiple(
            0,
            last_name=HL7String(1),
            first_name=HL7String(2),
            further_names=HL7String(3),
            prefix=HL7String(5),
            type_code=HL7String(7),
        ),
    )
    mother_maiden_name = HL7String(7)
    birthdate = HL7Date(8)
    sex = HL7String(9)
    adress = HprimList(
        12,
        basefield=HprimMultiple(
            0,
            street_adress=HL7String(1),
            other_designation=HL7String(2),
            city=HL7String(3),
            postal_code=HL7String(5),
            country=HL7String(6),
            type=HL7String(7),
            county_code=HL7String(9),
            effective_date=HL7Date(13),
            expiration_date=HL7Date(14),
        ),
    )
    telecommunication_home = HprimList(
        14,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    telecommunication_business = HprimList(
        15,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    primary_language = HL7String(16)
    marital_status = HL7String(17)
    # only in ITI-31
    account_number = HprimList(
        19,
        basefield=HprimMultiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HprimMultiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
            assigning_facility=HL7String(6),
            effective_date=HL7Date(7),
        ),
    )
    # not implemented in clarilab
    mother_identifier = HprimList(
        22,
        basefield=HprimMultiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HprimMultiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
            assigning_facility=HL7String(6),
            effective_date=HL7Date(7),
        ),
    )
    birth_order = HL7String(26)
    citizenship = HprimList(
        27,
        basefield=HprimMultiple(
            0, code=HL7String(1), label=HL7String(2), coding_system=HL7String(3)
        ),
    )
    identity_reliability_code = HL7String(33)
    last_update_date = HL7Date(34)
    last_update_facility = HL7String(35)

    patients = HL7Foreign("PD1", "patient")
    merges = HL7Foreign("MRG", "patient")


class HL7IHEPAMADTPD1(Segment):
    living_arrangement = HL7String(3)
    data_protection = HL7String(13)

    visits = HL7Foreign("PV1", "patient")
    roles = HL7Foreign("ROL", "patient")


class HL7IHEPAMADTROL(Segment):
    action_code = HL7String(3)
    role = HL7String(4)
    identity = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            identifier=HL7String(1),
            last_name=HL7String(2),
            first_name=HL7String(3),
            further_names=HL7String(4),
            prefix=HL7String(6),
            assigning_authority=HprimMultiple(
                9,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            name_type_code=HL7String(10),
            identifier_type_code=HL7String(13),
        ),
    )
    role_begin_date = HL7Date(6)
    role_end_date = HL7Date(7)

    kins = HL7Foreign("NK1", "patient")


class HL7IHEPAMADTNK1(Segment):
    relationship = HL7String(4)
    contact_role = HL7String(8)
    contact_identifier = HprimList(
        4,
        basefield=HprimMultiple(
            0,
            identifier=HL7String(1),
            assigning_authority=HprimMultiple(
                4,
                namespace_id=HL7String(1),
                universal_id=HL7String(2),
                universal_id_type=HL7String(3),
            ),
            type_code=HL7String(5),
            assigning_facility=HL7String(6),
            effective_date=HL7Date(7),
            assigning_jurisdiction=HL7String(9),
            assigning_department=HL7String(10),
        ),
    )

    visits = HL7Foreign("PV1", "patient")


class HL7IHEPAMADTPV1(Segment):
    patient_class = HL7String(3)
    point_of_care = HL7String((4, 1))
    room = HL7String((4, 2))
    bed = HL7String((4, 3))
    facility = HL7String((4, 4))
    bed_status = HL7String((4, 5))
    admission_type = HL7String(5)
    preadmit_number = HL7String(6)
    attending_doctor = HprimMultiple(
        8,
        identifier=HL7String(1),
        family_name=HL7String(2),
        given_name=HL7String(3),
        further_names=HL7String(4),
        prefix=HL7String(6),
        assigning_authority=HL7String(9),
        name_type_code=HL7String(10),
        identifier_type_code=HL7String(13),
    )
    referring_doctor = HprimMultiple(
        9,
        identifier=HL7String(1),
        family_name=HL7String(2),
        given_name=HL7String(3),
        further_names=HL7String(4),
        prefix=HL7String(6),
        assigning_authority=HL7String(9),
        name_type_code=HL7String(10),
        identifier_type_code=HL7String(13),
    )
    hospital_service = HL7String(12)
    admit_source = HL7String(15)
    vip_indicator = HL7String(17)
    admitting_doctor = HprimMultiple(
        18,
        identifier=HL7String(1),
        family_name=HL7String(2),
        given_name=HL7String(3),
        further_names=HL7String(4),
        prefix=HL7String(6),
        assigning_authority=HL7String(9),
        name_type_code=HL7String(10),
        identifier_type_code=HL7String(13),
    )
    visit_number = HL7String(20)
    financial_class = HL7String(21)
    charge_price_indicator = HL7String(22)
    courtesy_code = HL7String(23)
    discharge_disposition = HL7String(37)
    discharge_to_location = HL7String(38)
    account_status = HL7String(42)

    visits = HL7Foreign("PV2", "patient")
    demographics = HL7Foreign("ZFD", "patient")


class HL7IHEPAMADTPV2(Segment):
    accommodation_code = HL7String(3)
    visit_user_code = HL7String(8)
    expected_admit_date = HL7Date(9)
    expected_discharge_date = HL7Date(10)
    visit_description = HL7String(13)
    special_program_code = HL7String(19)
    visit_protection_indicator = HL7String(23)
    patient_charge_ajustment_code = HL7String(31)
    mode_of_arrival_code = HL7String(39)
    expected_loa_return_date = HL7Date(48)

    demographics = HL7Foreign("ZFD", "patient")


class HL7IHEPAMADTACC(Segment):
    accident_date = HL7Date(2)
    accident_code = HL7String((3, 1))
    accident_text = HL7String((3, 2))

    def to_string(self, separators=None):
        self.rang_segment = self.accident_date
        return super(HL7IHEPAMADTACC, self).to_string(separators=separators)


class HL7IHEPAMADTZFD(Segment):
    lunar_date = HL7Date(2)
    gestation_weeks = HL7String(3)
    sms_consent = HL7String(4)
    birthdate_modified = HL7String(5)
    identity_obtaining_mode = HL7String(6)
    insi_query_date = HL7Date(7)
    identity_document = HL7String(8)
    identity_document_expiration_date = HL7Date(9)

    def to_string(self, separators=None):
        self.rang_segment = self.lunar_date
        return super(HL7IHEPAMADTZFD, self).to_string(separators=separators)


class HL7IHEPAMADTZFS(Segment):
    legal_mode_of_care_id = HL7String((3, 1))
    legal_mode_of_care_label = HL7String((3, 2))
    legal_mode_of_care_start_date = HL7Date(4)
    legal_mode_of_care_end_date = HL7Date(5)
    legal_mode_of_care__action = HL7String(6)
    legal_mode_of_care = HL7String(7)
    legal_mode_of_care_rip_p_id = HL7String(8)
    comment = HL7String(9)


class HL7IHEPAMADTIN1(Segment):
    type_of_payer = HL7String(3)
    insurance_scheme = HL7String(4)
    coverage_period_beginning = HL7Date(13)
    coverage_period_end = HL7Date(14)
    patient_management_request = HL7String((15, 1))
    authorisation_date_of_delivery = HL7Date((15, 2))
    exemption_from_copayment = HL7String(16)
    insured_family_name = HL7String((17, 1))
    insured_given_name = HL7String((17, 2))
    insured_further_names = HL7String((17, 3))
    insured_prefix = HL7String((17, 5))
    insured_name_type_code = HL7String((17, 7))
    beneficiary_status = HL7String(18)
    insured_adress = HL7String((20, 1))
    insured_other_designation = HL7String((20, 2))
    insured_city = HL7String((20, 3))
    insured_postal_code = HL7String((20, 5))
    insured_country = HL7String((20, 6))
    insured_adress_type = HL7String((20, 7))
    insured_county_code = HL7String((20, 9))
    insured_adress_effective_date = HL7Date((20, 13))
    insured_adress_expiration_date = HL7Date((20, 14))
    third_party_payer = HL7String(21)
    insurance_nature = HL7String(32)
    management_code = HL7String(36)
    visit_coverage_number = HL7String(37)
    visit_coverage_date = HL7Date(37)
    chi_supporting_documentation_nature = HL7String(46)
    insurance_register_number = HL7String(50)


class HL7IHEPAMADTIN2(Segment):
    managed_services = HL7String(29)
    telecommunication = HprimList(
        64,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )


class HL7IHEPAMADTIN3(Segment):
    patient_medical_management_rate = HL7String(6)


class HL7IHEPAMADTGT1(Segment):
    guarantor_number = HL7String(3)
    guarantor_family_name = HL7String((4, 1))
    guarantor_given_name = HL7String((4, 2))
    guarantor_further_names = HL7String((4, 3))
    guarantor_prefix = HL7String((4, 5))
    guarantor_name_type_code = HL7String((4, 7))
    guarantor_adress = HL7String((6, 1))
    guarantor_other_designation = HL7String((6, 2))
    guarantor_city = HL7String((6, 3))
    guarantor_postal_code = HL7String((6, 5))
    guarantor_country = HL7String((6, 6))
    guarantor_adress_type = HL7String((6, 7))
    guarantor_county_code = HL7String((6, 9))
    guarantor_adress_effective_date = HL7Date((6, 13))
    guarantor_adress_expiration_date = HL7Date((6, 14))
    telecommunication_home = HprimList(
        7,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    telecommunication_business = HprimList(
        8,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    guarantor_type = HL7String(11)
    guarantor_relationship = HL7String(12)
    guarantor_ssn = HL7String(13)
    guarantor_date_begin = HL7Date(14)
    guarantor_date_end = HL7Date(15)
    guarantor_priority = HL7String(16)
    guarantor_employer_family_name = HL7String((17, 1))
    guarantor_employer_given_name = HL7String((17, 2))
    guarantor_employer_further_names = HL7String((17, 3))
    guarantor_employer_prefix = HL7String((17, 5))
    guarantor_employer_name_type_code = HL7String((17, 7))
    guarantor_employer_adress = HL7String((18, 1))
    guarantor_employer_other_designation = HL7String((18, 2))
    guarantor_employer_city = HL7String((18, 3))
    guarantor_employer_postal_code = HL7String((18, 5))
    guarantor_employer_country = HL7String((18, 6))
    guarantor_employer_adress_type = HL7String((18, 7))
    guarantor_employer_county_code = HL7String((18, 9))
    guarantor_employer_adress_effective_date = HL7Date((18, 13))
    guarantor_employer_adress_expiration_date = HL7Date((18, 14))
    telecommunication_employer = HprimList(
        19,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    guarantor_employee_id_number = HL7String(20)
    guarantor_employement_status = HL7String(21)
    guarantor_organization_status = HL7String(22)
    guarantor_billing_hold_flag = HL7String(23)
    guarantor_credit_rating_code = HL7String(24)
    guarantor_death_date = HL7Date(25)
    guarantor_death_date = HL7String(26)
    guarantor_charge_ajustement_code = HL7String(27)
    guarantor_household_annual_income = HL7String(28)
    guarantor_household_size = HL7String(29)
    guarantor_employer_id_number = HL7String(30)
    guarantor_martial_status_code = HL7String(31)
    guarantor_hire_effective_date = HL7Date(32)
    employement_stop_date = HL7Date(33)
    primary_language = HL7String(37)
    living_arrangement = HL7String(38)
    nationality = HL7String(44)
    contact_person_family_name = HL7String((46, 1))
    contact_person_given_name = HL7String((46, 2))
    contact_person_further_name = HL7String((46, 3))
    contact_person_prefix = HL7String((46, 5))
    contact_person_name_type_code = HL7String((46, 7))
    telecommunication_contact = HprimList(
        47,
        basefield=HprimMultiple(
            0,
            use_code=HL7String(2),
            equipement_type=HL7String(3),
            email=HL7String(4),
            phone_number=HL7String(12),
        ),
    )
    contact_reason = HL7String(48)
    contact_relationship = HL7String(49)
    job_totle = HL7String(50)
    job_code = HL7String(51)
    guarantor_employer_organization_name = HL7String(52)
    job_status = HL7String(54)
    guarantor_financial_class = HL7String(55)
    guarantor_birth_place = HL7String(57)
    vip_indicator = HL7String(58)


class HL7IHEPAMADTOBX(Segment):
    value_type = HL7String(3)
    observation_identifier = HL7String((4, 1))
    observation_text = HL7String((4, 2))
    coding_system = HL7String((4, 3))
    observation_value = HL7String(6)
    units = HL7String(7)
    observation_result_status = HL7String(12)
    observation_date = HL7Date(15)
    validator_id = HL7String((17, 1))
    validator_family_name = HL7String((17, 2))
    validator_given_name = HL7String((17, 3))
    validator_prefix = HL7String((17, 6))
    validator_assigning_authority = HL7String((17, 9))
    validator_identifier_type_code = HL7String((17, 13))


class HL7IHEPAMADTMRG(Segment):
    id_number = HL7String((2, 1))
    check_digit = HL7String((2, 2))
    check_digit_scheme = HL7String((2, 3))
    assigning_authority = HL7String((2, 4))
    identifier_type_code = HL7String((2, 5))
    assigning_facility = HL7String((2, 6))
    effective_date = HL7Date((2, 7))
    expiration_date = HL7Date((2, 8))
    assigning_jurisdiction = HL7String((2, 9))
    assigning_department = HL7String((2, 10))

    def to_string(self, separators=None):
        self.rang_segment = self.id_number
        return super(HL7IHEPAMADTMRG, self).to_string(separators=separators)


class Driver(AstmMessageV2):
    SPEC = {"code": "hl7_ihe_pam_adt", "label": "HL7 IHE PAM ADT"}
    ENCODING = "utf-8"

    SEGMENT_CLASS_MAP = {
        "MSH": HL7IHEPAMADTMSH,
        "EVN": HL7IHEPAMADTEVN,
        "PID": HL7IHEPAMADTPID,
        "PD1": HL7IHEPAMADTPD1,
        "ROL": HL7IHEPAMADTROL,
        "NK1": HL7IHEPAMADTNK1,
        "PV1": HL7IHEPAMADTPV1,
        "PV2": HL7IHEPAMADTPV2,
        "ACC": HL7IHEPAMADTACC,
        "ZFD": HL7IHEPAMADTZFD,
        "ZFS": HL7IHEPAMADTZFS,
        "IN1": HL7IHEPAMADTIN1,
        "IN2": HL7IHEPAMADTIN2,
        "IN3": HL7IHEPAMADTIN3,
        "GT1": HL7IHEPAMADTGT1,
        "OBX": HL7IHEPAMADTOBX,
        "MRG": HL7IHEPAMADTMRG,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def decode_message(self, message):
        if message.trigger_event not in ("A28", "A31", "A47", "A40"):
            raise NotImplementedError

        worklist = Worklist(
            context="order",
            sender_id=message.sending_application,
            receiver_id=message.receiving_application,
        )

        for seg_evn in message.events:
            for seg_pid in seg_evn.patients:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id="",
                    birthdate=seg_pid.birthdate,
                    sex=seg_pid.sex,
                    language=seg_pid.primary_language,
                    marital_status=seg_pid.marital_status,
                    birth_order=seg_pid.birth_order,
                    identity_update_date=seg_pid.last_update_date,
                )

                ins_number = ""
                ipp = ""
                oid = ""
                identity_status = ""
                for p in seg_pid.patient_identifier:
                    if p.type_code == "INS-NIR":
                        ins_number = p.identifier
                        oid = p.assigning_authority.universal_id
                        if seg_pid.identity_reliability_code == "VALI":
                            identity_status = "Qualifiée"
                    if p.type_code == "PI":
                        ipp = p.identifier
                        if (
                            seg_pid.identity_reliability_code == "PROV"
                            and identity_status == ""
                        ):
                            identity_status = "Provisoire"

                last_name = ""
                used_last_name = ""
                first_name = ""
                used_first_name = ""
                further_names = []
                prefix = ""
                type_code = ""
                for i in seg_pid.identity:
                    if i.type_code == "L":
                        last_name = i.last_name
                        first_name = i.first_name
                        if i.further_names:
                            for name in i.further_names.split(" "):
                                further_names.append(name)
                        prefix = i.prefix
                        type_code = "normale"
                    elif i.type_code == "D":
                        used_last_name = i.last_name
                        used_first_name = i.first_name
                        if type_code == "":
                            type_code = "douteuse"

                adress = ""
                birthplace = ""
                birthplace_code = ""
                city = ""
                postal_code = ""
                county_code = ""
                country = ""
                effective_date = None
                expiration_date = None
                for a in seg_pid.adress:
                    if a.type == "BDL":
                        birthplace = a.city
                        birthplace_code = a.county_code
                    elif a.type == "H":
                        adress = "{}~{}".format(a.street_adress, a.other_designation)
                        city = a.city
                        county_code = a.county_code
                        country = a.country
                        postal_code = a.postal_code
                        effective_date = a.effective_date
                        expiration_date = a.expiration_date

                phone = ""
                email = ""
                for t in seg_pid.telecommunication_home:
                    if t.phone_number != "":
                        phone = t.phone_number
                    if t.email != "":
                        email = t.email

                citizenships = []
                for c in seg_pid.citizenship:
                    if c.code:
                        citizenships.append(c.code)

                sample.update(
                    patient_id=ipp,
                    ins_number=ins_number,
                    oid=oid,
                    identity_status=identity_status,
                    last_name=last_name,
                    used_last_name=used_last_name,
                    first_name=first_name,
                    used_first_name=used_first_name,
                    further_names=further_names,
                    identity_attribut=type_code,
                    title=prefix,
                    address=adress,
                    city=city,
                    county_code=county_code,
                    country=country,
                    postal_code=postal_code,
                    adress_effective_date=effective_date,
                    adress_expiration_date=expiration_date,
                    birthplace=birthplace,
                    birthplace_code=birthplace_code,
                    phone=phone,
                    email=email,
                    citizenships=citizenships,
                )

                for seg_pd1 in seg_pid.patients:
                    sample.update(
                        living_arrangement=seg_pd1.living_arrangement,
                        data_protection=seg_pd1.data_protection,
                    )
                    for seg_pv1 in seg_pd1.visits:
                        sample.update(patient_class=seg_pv1.patient_class)
                        for seg_zfd in seg_pv1.demographics:
                            sample.update(
                                birthdate_modified=seg_zfd.birthdate_modified,
                                identity_obtaining_mode=seg_zfd.identity_obtaining_mode,
                                insi_query_date=seg_zfd.insi_query_date,
                                identity_document=seg_zfd.identity_document,
                                identity_document_expiration_date=seg_zfd.identity_document_expiration_date,
                            )

        return worklist
